package tests;

import java.io.*;

public class Client {

    private static Tests tests;

    public static void initORB(String [] args) {

	if (args.length != 1){
	    System.out.println("Use: jaco tests.Client <file_containing_ior>");
	    System.exit(1);
	}

	String name = "Tests";

	try
	{

	    /* Precisamos do ORB do JacORB, pois Message Interceptors sao proprietarios */
	    jacorb.orb.ORB orb = (jacorb.orb.ORB)org.omg.CORBA.ORB.init(args,null);

	    // Read IOR-string form args[0] 
	    System.out.println("Not using the NameService");

	    try {
		BufferedReader br = new BufferedReader 
		    (new FileReader (args[0]));
		
		tests = TestsHelper.narrow(orb.string_to_object(br.readLine()));
		br.close();
	    }
	    catch (IOException e) {
		System.out.println ("Client: IOError, couldn't read IOR: " + e);
		System.exit(1);
	    }
	    
	    
	    // Registrando o interceptador
	    System.out.println ("Registrando Client Interceptor..");
	    
	    ClientMessageInterceptor cmi = new ClientMessageInterceptor();
	    orb.addInterceptor (cmi);
	    

	} catch (Exception e) {
	    System.out.println(e);
	    e.printStackTrace();
	    System.exit(1);
	}

    }

    public static void main(String [] args) {

	// Inicializacao do ORB
	initORB(args);


	/* Lendo as opcoes do usuario */
	System.out.println ("--------------------------------------------");
	System.out.println ("Menu de opcoes:");
	System.out.println ("(1) string toUpper (in string s) (String de entrada em texto.txt)");
	System.out.println ("(2) string retString (in long i) (String de entrada do servidor em string_resposta_server.txt");
	System.out.println ("(3) OctetSeq doNothing (in OctetSeq i) (Buffer de entrada em entrada_bytes");
	System.out.println ("--------------------------------------------");
	System.out.print ("Digite sua opcao: ");

	int opcao = -1;

	try {
	    BufferedReader reader = new BufferedReader (new InputStreamReader 
							(System.in));
	    opcao = Integer.parseInt (reader.readLine());
	} catch (IOException e) {
	    System.out.println ("Erro na leitura de opcao do usuario: " + e);
	    System.exit(1);
	}
	
	
	/** Chamando cada metodo, de acordo com a opcao do usuario **/
	switch (opcao) {

	case 1:

	    callToUpper();

	    break;

	case 2:

	    callRetString();

	    break;

	case 3:

	    callDoNothing();

	    break;

	default: 
	    System.out.println ("Opcao invalida");
	    System.exit (1);

	} // of switch 



    }


    /**** callToUpper ****/
    private static void callToUpper () {

	Crono crono = new Crono ("toUpper");
	
	/* Lendo o arquivo de entrada */

	try {
	    System.out.print ("Lendo texto.txt................");
	    BufferedReader br = new BufferedReader (new FileReader ("texto.txt"));
	    
	    String text = "";
	    String tmp = br.readLine();
	    while (tmp != null) {
		text += tmp + "\n";
		tmp = br.readLine();
		}

	    System.out.println (".......................done");

	    System.out.print ("Chamando toUpper.......................");
	    crono.start();
	    String response = tests.toUpper (text);
	    crono.stop();
	    System.out.println ("......................... done.");

	    // Debug 
	    System.out.print ("Salvando resposta...............................");
	    PrintWriter pw = new PrintWriter (new FileWriter ("client_mensagem_devolvida"));
	    pw.print (response);
	    pw.close();
	    System.out.println (".................. done (resposta salva em client_mensagem_devolvida");

	    System.out.println (crono);

	} catch  (IOException e) {
	    System.out.println ("Erro de IO: " + e);
	    e.printStackTrace();
	}

    } // of callToUpper


    /**** callRetString ****/
    private static void callRetString () {

	Crono crono = new Crono ("retString");

	System.out.print ("Chamando retString.......................");
	crono.start();
	String response = tests.retString (22);
	crono.stop();
	System.out.println (".................. done.");
	

	// Debug 
	try {
	    System.out.print ("Salvando resposta...............................");
	    PrintWriter pw = new PrintWriter (new FileWriter ("client_mensagem_devolvida"));
	    pw.print (response);
	    pw.close();
	    System.out.println (".................. done (resposta salva em client_mensagem_devolvida");

	    System.out.println (crono);
	    
	} catch  (IOException e) {
	    System.out.println ("Erro de IO: " + e);
	    e.printStackTrace();
	}


	
    } // of callRetString

    
    /**** callDoNothing ****/
    private static void callDoNothing () {

	Crono crono = new Crono ("toUpper");

	FileInputStream in = null;
	
	try {

	    System.out.print ("Lendo entrada_bytes................");
	    in = new FileInputStream ("entrada_bytes");
	 
	    ByteArrayOutputStream out = new ByteArrayOutputStream();
	    
	    byte[] temp = new byte[300000];
	    
	    int bytesRead = 0;
	    while (bytesRead != -1) {
		
		bytesRead = in.read (temp);
		if (bytesRead != -1)
		    out.write (temp, 0, bytesRead);
		
	    }
	    
	    out.close();

	    System.out.println (".......................done");

	    // Fazendo a chamada
	    System.out.print ("Chamando doNothing.......................");
	    crono.start();
	    byte[] resposta = tests.doNothing (out.toByteArray());
	    crono.stop();
	    System.out.println (".................. done.");


	    // Imprimindo a resposta
	    System.out.print ("Salvando resposta...............................");
	    FileOutputStream resp = new FileOutputStream ("client_buffer_devolvido");
	    resp.write (resposta);
	    resp.close();	    
	    System.out.println (".................. done (resposta salva em client_buffer_devolvido");

	    System.out.println (crono);

	}
	catch (IOException e) {
	    System.out.println ("Erro de IO " + e);
	    System.exit(0);
	}
	
	

    } // of callDoNothing


} // of class
